# -*- coding: utf-8 -*-

"""
test_custom_extension_in_hooks.

Tests to ensure custom cookiecutter extensions are properly made available to
pre- and post-gen hooks.
"""

import codecs
import os
from _collections import OrderedDict

import pytest

from cookiecutter import main


@pytest.fixture(
    params=['custom-extension-pre', 'custom-extension-post'],
    ids=['pre_gen_hook', 'post_gen_hook'],
)
def template(request):
    """Fixture. Allows to split pre and post hooks test directories."""
    return 'test-extensions/' + request.param


@pytest.fixture
def output_dir(tmpdir):
    """Fixture. Create and return custom temp directory for test."""
    return str(tmpdir.mkdir('hello'))


@pytest.fixture(autouse=True)
def modify_syspath(monkeypatch):
    """Fixture. Make sure that the custom extension can be loaded."""
    monkeypatch.syspath_prepend('test-extensions/hello_extension')


def test_hook_with_extension(monkeypatch, template, output_dir):
    """Verify custom Jinja2 extension correctly work in hooks and file rendering.

    Each file in hooks has simple tests inside and will raise error if not
    correctly rendered.
    """
    monkeypatch.chdir(os.path.abspath(os.path.dirname(__file__)))

    context = main.cookiecutter(
        template,
        no_input=True,
        output_dir=output_dir,
        extra_context={'project_slug': 'foobar', 'name': 'Cookiemonster'},
    )

    readme_file = os.path.join(os.path.join(output_dir, 'foobar', 'README.rst'))
    with codecs.open(readme_file, encoding='utf8') as f:
        readme = f.read().strip()
    assert readme == 'Hello Cookiemonster!'

    assert type(context) == OrderedDict



# -*- coding: utf-8 -*-

# flake8: noqa

"""Sample post-gen hook for testing that custom extensions are available and exposed methods are callable."""
import sys

if '{% hello cookiecutter.name %}' == 'Hello Cookiemonster!':
    sys.exit(0)
else:
    sys.exit(1)



# -*- coding: utf-8 -*-

"""
test_abort_generate_on_hook_error.

Tests to ensure cookiecutter properly exits with a non-zero exit code whenever
errors occur in (optional) pre- or pos-gen hooks.
"""

import pytest

from cookiecutter import exceptions, generate


@pytest.mark.parametrize(
    ("abort_pre_gen", "abort_post_gen"),
    (("yes", "no"), ("no", "yes")),
    ids=("pre_gen_hook_raises_error", "post_gen_hook_raises_error"),
)
@pytest.mark.usefixtures("clean_system")
def test_hooks_raises_errors(tmpdir, abort_pre_gen, abort_post_gen):
    """Verify pre- and pos-gen errors raises correct error code from script.

    This allows developers to make different error codes in their code,
    for different errors.
    """
    context = {
        "cookiecutter": {
            "repo_dir": "foobar",
            "abort_pre_gen": abort_pre_gen,
            "abort_post_gen": abort_post_gen,
        }
    }

    with pytest.raises(exceptions.FailedHookException) as error:
        generate.generate_files(
            repo_dir="tests/hooks-abort-render", context=context, output_dir=str(tmpdir)
        )
        assert error.value.code == 5
    assert not tmpdir.join("foobar").isdir()



# -*- coding: utf-8 -*-

"""Tests for all supported cookiecutter template repository locations."""

import pytest

from cookiecutter.config import BUILTIN_ABBREVIATIONS
from cookiecutter.repository import is_zip_file, is_repo_url, expand_abbreviations


@pytest.fixture(
    params=[
        '/path/to/zipfile.zip',
        'https://example.com/path/to/zipfile.zip',
        'http://example.com/path/to/zipfile.zip',
    ]
)
def zipfile(request):
    """Fixture. Represent possible paths to zip file."""
    return request.param


def test_is_zip_file(zipfile):
    """Verify is_repo_url works."""
    assert is_zip_file(zipfile) is True


@pytest.fixture(
    params=[
        'gitolite@server:team/repo',
        'git@github.com:audreyr/cookiecutter.git',
        'https://github.com/audreyr/cookiecutter.git',
        'git+https://private.com/gitrepo',
        'hg+https://private.com/mercurialrepo',
        'https://bitbucket.org/pokoli/cookiecutter.hg',
        'file://server/path/to/repo.git',
    ]
)
def remote_repo_url(request):
    """Fixture. Represent possible URI to different repositories types."""
    return request.param


def test_is_repo_url_for_remote_urls(remote_repo_url):
    """Verify is_repo_url works."""
    assert is_repo_url(remote_repo_url) is True


@pytest.fixture(
    params=[
        '/audreyr/cookiecutter.git',
        '/home/audreyr/cookiecutter',
        (
            'c:\\users\\foo\\appdata\\local\\temp\\1\\pytest-0\\'
            'test_default_output_dir0\\template'
        ),
    ]
)
def local_repo_url(request):
    """Fixture. Represent possible paths to local resources."""
    return request.param


def test_is_repo_url_for_local_urls(local_repo_url):
    """Verify is_repo_url works."""
    assert is_repo_url(local_repo_url) is False


def test_expand_abbreviations():
    """Validate `repository.expand_abbreviations` correctly translate url."""
    template = 'gh:audreyr/cookiecutter-pypackage'

    # This is not a valid repo url just yet!
    # First `repository.expand_abbreviations` needs to translate it
    assert is_repo_url(template) is False

    expanded_template = expand_abbreviations(template, BUILTIN_ABBREVIATIONS)
    assert is_repo_url(expanded_template) is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""cookiecutter distutils configuration."""

import os
import io
import sys

from setuptools import setup

version = "1.7.2.7"

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

with io.open('README.md', 'r', encoding='utf-8') as readme_file:
    readme = readme_file.read()

requirements = [
    'binaryornot>=0.4.4',
    'Jinja2<3.0.0',
    'click>=7.0',
    'poyo>=0.5.0',
    'jinja2-time>=0.2.0',
    'python-slugify>=4.0.0',
    'requests>=2.23.0',
    'six>=1.10',
    'MarkupSafe<2.0.0',
    'PyInquirer>=1.0.3',
    'PyYAML>=5.3',
]

if sys.argv[-1] == 'readme':
    print(readme)
    sys.exit()

setup(
    name='nukikata',
    version=version,
    description=(
        'Fork of cookiecutter - https://github.com/cookiecutter/cookiecutter '
        'The most popular command-line utility to create projects from project '
        'templates, e.g. creating a Python package project from a '
        'Python package project template.'
    ),
    long_description=readme,
    long_description_content_type='text/markdown',
    author='Original Author - Audrey Roy, Fork by Rob Cannon',
    author_email='rob.cannon@insightinfrastructure.com',
    url='https://github.com/insight-infrastructure/nukikata',
    packages=['cookiecutter'],
    package_dir={'cookiecutter': 'cookiecutter'},
    entry_points={'console_scripts': ['nukikata = cookiecutter.__main__:main']},
    include_package_data=True,
    python_requires='>=3.5',
    install_requires=requirements,
    extras_require={':python_version<"3.3"': ['whichcraft>=0.4.0']},
    license='BSD',
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development",
    ],
    keywords=(
        'cookiecutter, Python, projects, project templates, Jinja2, '
        'skeleton, scaffolding, project directory, setup.py, package, '
        'packaging, insight, insight data science, insight infrastructure, '
        'cli'
    ),
)



# -*- coding: utf-8 -*-

"""Operator plugin that inherits a base class and is made available through `type`."""
from __future__ import unicode_literals
from __future__ import print_function

import os
import logging

from cookiecutter.operators import BaseOperator

logger = logging.getLogger(__name__)


class ListdirOperator(BaseOperator):
    """Operator for PyInquirer type prompts."""

    type = 'listdir'

    def __init__(self, operator_dict, context=None, no_input=False):
        """Initialize operator."""
        super(ListdirOperator, self).__init__(
            operator_dict=operator_dict, context=context, no_input=no_input
        )
        # Defaulting to run inline
        self.post_gen_operator = (
            self.operator_dict['delay'] if 'delay' in self.operator_dict else False
        )
        self.ignore_hidden_files = (
            self.operator_dict['ignore_hidden_files']
            if 'ignore_hidden_files' in self.operator_dict
            else False
        )

    def execute(self):
        """Run the operator."""  # noqa
        if 'directory' in self.operator_dict:
            files = os.listdir(self.operator_dict['directory'])
            if self.ignore_hidden_files:
                return [f for f in files if not f.startswith('.')]
            else:
                return files

        elif isinstance(self.operator_dict['directories'], list):
            # If instance is a list, return a dict with the keys as the items in list
            contents = {}
            for i in self.operator_dict['directories']:
                contents[i] = os.listdir(i)
            return contents
        else:
            raise NotImplementedError(
                "Have not implemented dict input to "
                "`directories` for `type` 'listdir'"
            )



# -*- coding: utf-8 -*-

"""Operator plugin that inherits a base class and is made available through `type`."""
from __future__ import unicode_literals
from __future__ import print_function

import logging
from pprint import pprint

from cookiecutter.operators import BaseOperator

logger = logging.getLogger(__name__)


class PrintOperator(BaseOperator):
    """Operator for printing an input and returning the output."""

    type = 'print'

    def __init__(self, operator_dict, context=None, no_input=False):
        """Initialize operator."""  # noqa
        super(PrintOperator, self).__init__(
            operator_dict=operator_dict, context=context, no_input=no_input
        )

    def execute(self):
        """Print the statement."""
        print(self.operator_dict['statement'])
        return self.operator_dict['statement']


class PprintOperator(BaseOperator):
    """Operator for pretty printing an input and returning the output."""

    type = 'pprint'

    def __init__(self, operator_dict, context=None, no_input=False):
        """Initialize operator."""  # noqa
        super(PprintOperator, self).__init__(
            operator_dict=operator_dict, context=context, no_input=no_input
        )

    def execute(self):
        """Run the operator."""  # noqa
        pprint(self.operator_dict['statement'])
        return self.operator_dict['statement']



# -*- coding: utf-8 -*-

"""Tests dict input objects for `cookiecutter.prompt` module."""

# from helpers import create_example_fixture, keys
# import os
#
# FIXTURES = ['checkbox_bad.yaml']
#
# example_app = create_example_fixture(os.path.join(os.path.dirname(__file__),
#                                                   'checkbox.py'))
#
# def test_checkbox(example_app):
#     # TODO: get psuedo terminal working to enter in commands
#     x = example_app.write(keys.ENTER)
#     print()
#     x = example_app.write(keys.ENTER)
#     x = example_app.write(keys.ENTER)
#     print()


# class TestPromptHookDicts(object):
#     """Class to test all the dict input objects."""
#
#     @pytest.mark.parametrize("input_dicts,invalid", FIXTURES)
#     def test_hook_checkbox(self, monkeypatch):
#         """Verify `prompt_for_config` call `read_user_variable` on dict request."""
#
#         monkeypatch.setattr(
#             'cookiecutter.prompt.read_user_dict',
#             lambda var, default: {"key": "value", "integer": 37},
#         )
#         context = {'cookiecutter': {'details': {}}}
#
#         cookiecutter_dict = prompt.prompt_for_config(context)
#         assert cookiecutter_dict == {'details': {'key': u'value', 'integer': 37}}



# -*- coding: utf-8 -*-

"""Operator plugin that inherits a base class and is made available through `type`."""
from __future__ import unicode_literals
from __future__ import print_function

import logging

from cookiecutter.operators import BaseOperator

logger = logging.getLogger(__name__)


class StatOperator(BaseOperator):
    """Operator for registering a variable based on an input. Useful with rendering."""

    type = 'stat'

    def __init__(self, operator_dict, context=None, no_input=False):
        """Initialize operator."""  # noqa
        super(StatOperator, self).__init__(
            operator_dict=operator_dict, context=context, no_input=no_input
        )

    def execute(self):
        """."""  # noqa
        return self.operator_dict['input']



# -*- coding: utf-8 -*-

"""Tests for `generate_files` function and related errors raising.

Use the global clean_system fixture and run additional teardown code to remove
some special folders.

For a better understanding - order of fixture calls:
clean_system setup code
remove_additional_folders setup code
remove_additional_folders teardown code
clean_system teardown code
"""

from __future__ import unicode_literals

import io
import os

import pytest
from binaryornot.check import is_binary
from cookiecutter import exceptions, generate, utils


@pytest.mark.parametrize('invalid_dirname', ['', '{foo}', '{{foo', 'bar}}'])
def test_ensure_dir_is_templated_raises(invalid_dirname):
    """Verify `ensure_dir_is_templated` raises on wrong directories names input."""
    with pytest.raises(exceptions.NonTemplatedInputDirException):
        generate.ensure_dir_is_templated(invalid_dirname)


@pytest.fixture(scope='function')
def remove_additional_folders():
    """Remove some special folders which are created by the tests."""
    yield
    if os.path.exists('inputpizzä'):
        utils.rmtree('inputpizzä')
    if os.path.exists('inputgreen'):
        utils.rmtree('inputgreen')
    if os.path.exists('inputbinary_files'):
        utils.rmtree('inputbinary_files')
    if os.path.exists('tests/custom_output_dir'):
        utils.rmtree('tests/custom_output_dir')
    if os.path.exists('inputpermissions'):
        utils.rmtree('inputpermissions')


# Removed because now we don't want to force actual rendering now that we have more
# to do...
# @pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
# def test_generate_files_nontemplated_exception():
#     """
#     Verify `generate_files` raises when no directories to render exist.
#
#     Note: Check `tests/test-generate-files-nontemplated` location to understand.
#     """
#     with pytest.raises(exceptions.NonTemplatedInputDirException):
#         generate.generate_files(
#             context={'cookiecutter': {'food': 'pizza'}},
#             repo_dir='tests/test-generate-files-nontemplated',
#         )


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files():
    """Verify directory name correctly rendered with unicode containing context."""
    generate.generate_files(
        context={'cookiecutter': {'food': 'pizzä'}},
        repo_dir='tests/test-generate-files',
    )

    simple_file = 'inputpizzä/simple.txt'
    assert os.path.isfile(simple_file)

    simple_text = io.open(simple_file, 'rt', encoding='utf-8').read()
    assert simple_text == u'I eat pizzä'


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_with_trailing_newline():
    """Verify new line not removed by templating engine after folder generation."""
    generate.generate_files(
        context={'cookiecutter': {'food': 'pizzä'}},
        repo_dir='tests/test-generate-files',
    )

    newline_file = 'inputpizzä/simple-with-newline.txt'
    assert os.path.isfile(newline_file)

    with io.open(newline_file, 'r', encoding='utf-8') as f:
        simple_text = f.read()
    assert simple_text == u'I eat pizzä\n'


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_binaries():
    """Verify binary files created during directory generation."""
    generate.generate_files(
        context={'cookiecutter': {'binary_test': 'binary_files'}},
        repo_dir='tests/test-generate-binaries',
    )

    assert is_binary('inputbinary_files/logo.png')
    assert is_binary('inputbinary_files/.DS_Store')
    assert not is_binary('inputbinary_files/readme.txt')
    assert is_binary('inputbinary_files/some_font.otf')
    assert is_binary('inputbinary_files/binary_files/logo.png')
    assert is_binary('inputbinary_files/binary_files/.DS_Store')
    assert not is_binary('inputbinary_files/binary_files/readme.txt')
    assert is_binary('inputbinary_files/binary_files/some_font.otf')
    assert is_binary('inputbinary_files/binary_files/binary_files/logo.png')


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_absolute_path():
    """Verify usage of `abspath` does not change files generation behaviour."""
    generate.generate_files(
        context={'cookiecutter': {'food': 'pizzä'}},
        repo_dir=os.path.abspath('tests/test-generate-files'),
    )
    assert os.path.isfile('inputpizzä/simple.txt')


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_output_dir():
    """Verify `output_dir` option for `generate_files` changing location correctly."""
    os.mkdir('tests/custom_output_dir')
    project_dir = generate.generate_files(
        context={'cookiecutter': {'food': 'pizzä'}},
        repo_dir=os.path.abspath('tests/test-generate-files'),
        output_dir='tests/custom_output_dir',
    )
    assert os.path.isfile('tests/custom_output_dir/inputpizzä/simple.txt')
    assert project_dir == os.path.abspath('tests/custom_output_dir/inputpizzä')


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_permissions():
    """Verify generates files respect source files permissions.

    simple.txt and script.sh should retain their respective 0o644 and 0o755
    permissions.
    """
    generate.generate_files(
        context={'cookiecutter': {'permissions': 'permissions'}},
        repo_dir='tests/test-generate-files-permissions',
    )

    assert os.path.isfile('inputpermissions/simple.txt')

    # simple.txt should still be 0o644
    tests_simple_file = os.path.join(
        'tests',
        'test-generate-files-permissions',
        'input{{cookiecutter.permissions}}',
        'simple.txt',
    )
    tests_simple_file_mode = os.stat(tests_simple_file).st_mode & 0o777

    input_simple_file = os.path.join('inputpermissions', 'simple.txt')
    input_simple_file_mode = os.stat(input_simple_file).st_mode & 0o777
    assert tests_simple_file_mode == input_simple_file_mode

    assert os.path.isfile('inputpermissions/script.sh')

    # script.sh should still be 0o755
    tests_script_file = os.path.join(
        'tests',
        'test-generate-files-permissions',
        'input{{cookiecutter.permissions}}',
        'script.sh',
    )
    tests_script_file_mode = os.stat(tests_script_file).st_mode & 0o777

    input_script_file = os.path.join('inputpermissions', 'script.sh')
    input_script_file_mode = os.stat(input_script_file).st_mode & 0o777
    assert tests_script_file_mode == input_script_file_mode


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_with_overwrite_if_exists_with_skip_if_file_exists():
    """Verify `skip_if_file_exist` has priority over `overwrite_if_exists`."""
    simple_file = 'inputpizzä/simple.txt'
    simple_with_new_line_file = 'inputpizzä/simple-with-newline.txt'

    os.makedirs('inputpizzä')
    with open(simple_file, 'w') as f:
        f.write('temp')

    generate.generate_files(
        context={'cookiecutter': {'food': 'pizzä'}},
        repo_dir='tests/test-generate-files',
        overwrite_if_exists=True,
        skip_if_file_exists=True,
    )

    assert os.path.isfile(simple_file)
    assert os.path.isfile(simple_with_new_line_file)

    simple_text = io.open(simple_file, 'rt', encoding='utf-8').read()
    assert simple_text == u'temp'


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_with_skip_if_file_exists():
    """Verify existed files not removed if error raised with `skip_if_file_exists`."""
    simple_file = 'inputpizzä/simple.txt'
    simple_with_new_line_file = 'inputpizzä/simple-with-newline.txt'

    os.makedirs('inputpizzä')
    with open(simple_file, 'w') as f:
        f.write('temp')

    with pytest.raises(exceptions.OutputDirExistsException):
        generate.generate_files(
            context={'cookiecutter': {'food': 'pizzä'}},
            repo_dir='tests/test-generate-files',
            skip_if_file_exists=True,
        )

    assert os.path.isfile(simple_file)
    assert not os.path.exists(simple_with_new_line_file)

    simple_text = io.open(simple_file, 'rt', encoding='utf-8').read()
    assert simple_text == u'temp'


@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
def test_generate_files_with_overwrite_if_exists():
    """Verify overwrite_if_exists overwrites old files."""
    simple_file = 'inputpizzä/simple.txt'
    simple_with_new_line_file = 'inputpizzä/simple-with-newline.txt'

    os.makedirs('inputpizzä')
    with open(simple_file, 'w') as f:
        f.write('temp')

    generate.generate_files(
        context={'cookiecutter': {'food': 'pizzä'}},
        repo_dir='tests/test-generate-files',
        overwrite_if_exists=True,
    )

    assert os.path.isfile(simple_file)
    assert os.path.isfile(simple_with_new_line_file)

    simple_text = io.open(simple_file, 'rt', encoding='utf-8').read()
    assert simple_text == u'I eat pizzä'


@pytest.fixture
def undefined_context():
    """Fixture. Populate context variable for future tests."""
    return {
        'cookiecutter': {'project_slug': 'testproject', 'github_username': 'hackebrot'}
    }


def test_raise_undefined_variable_file_name(tmpdir, undefined_context):
    """Verify correct error raised when file name cannot be rendered."""
    output_dir = tmpdir.mkdir('output')

    with pytest.raises(exceptions.UndefinedVariableInTemplate) as err:
        generate.generate_files(
            repo_dir='tests/undefined-variable/file-name/',
            output_dir=str(output_dir),
            context=undefined_context,
        )
    error = err.value
    assert "Unable to create file '{{cookiecutter.foobar}}'" == error.message
    assert error.context == undefined_context

    assert not output_dir.join('testproject').exists()


def test_raise_undefined_variable_file_name_existing_project(tmpdir, undefined_context):
    """Verify correct error raised when file name cannot be rendered."""
    output_dir = tmpdir.mkdir('output')

    output_dir.join('testproject').mkdir()

    with pytest.raises(exceptions.UndefinedVariableInTemplate) as err:
        generate.generate_files(
            repo_dir='tests/undefined-variable/file-name/',
            output_dir=str(output_dir),
            context=undefined_context,
            overwrite_if_exists=True,
        )
    error = err.value
    assert "Unable to create file '{{cookiecutter.foobar}}'" == error.message
    assert error.context == undefined_context

    assert output_dir.join('testproject').exists()


def test_raise_undefined_variable_file_content(tmpdir, undefined_context):
    """Verify correct error raised when file content cannot be rendered."""
    output_dir = tmpdir.mkdir('output')

    with pytest.raises(exceptions.UndefinedVariableInTemplate) as err:
        generate.generate_files(
            repo_dir='tests/undefined-variable/file-content/',
            output_dir=str(output_dir),
            context=undefined_context,
        )
    error = err.value
    assert "Unable to create file 'README.rst'" == error.message
    assert error.context == undefined_context

    assert not output_dir.join('testproject').exists()


def test_raise_undefined_variable_dir_name(tmpdir, undefined_context):
    """Verify correct error raised when directory name cannot be rendered."""
    output_dir = tmpdir.mkdir('output')

    with pytest.raises(exceptions.UndefinedVariableInTemplate) as err:
        generate.generate_files(
            repo_dir='tests/undefined-variable/dir-name/',
            output_dir=str(output_dir),
            context=undefined_context,
        )
    error = err.value

    directory = os.path.join('testproject', '{{cookiecutter.foobar}}')
    msg = "Unable to create directory '{}'".format(directory)
    assert msg == error.message

    assert error.context == undefined_context

    assert not output_dir.join('testproject').exists()


def test_raise_undefined_variable_dir_name_existing_project(tmpdir, undefined_context):
    """Verify correct error raised when directory name cannot be rendered."""
    output_dir = tmpdir.mkdir('output')

    output_dir.join('testproject').mkdir()

    with pytest.raises(exceptions.UndefinedVariableInTemplate) as err:
        generate.generate_files(
            repo_dir='tests/undefined-variable/dir-name/',
            output_dir=str(output_dir),
            context=undefined_context,
            overwrite_if_exists=True,
        )
    error = err.value

    directory = os.path.join('testproject', '{{cookiecutter.foobar}}')
    msg = "Unable to create directory '{}'".format(directory)
    assert msg == error.message

    assert error.context == undefined_context

    assert output_dir.join('testproject').exists()


def test_raise_undefined_variable_project_dir(monkeypatch, tmpdir):
    """Verify correct error raised when directory name cannot be rendered."""
    monkeypatch.chdir(os.path.abspath(os.path.dirname(__file__)))
    output_dir = tmpdir.mkdir('output')

    with pytest.raises(exceptions.UndefinedVariableInTemplate) as err:
        generate.generate_files(
            repo_dir='undefined-variable/dir-name/',
            output_dir=str(output_dir),
            context={'cookiecutter': {'stuff': 'things'}},
        )
    error = err.value
    msg = "Unable to create project directory '{{cookiecutter.project_slug}}'"
    assert msg == error.message
    assert error.context == {'cookiecutter': {'stuff': 'things'}}

    assert not output_dir.join('testproject').exists()
